@@ -1,7 +1,7 @@
 # This file registers all distributions when the package is loaded.
 .onAttach <- function(libname, pkgname) {
 
-  packageStartupMessage("Loading nimbleEcology. Registering the following distributions:\n ",
+  packageStartupMessage("Loading nimbleEcology. Registering multiple variants of the following distributions:\n ",
                         "dOcc", ", dDynOcc", ", dCJS", ", dHMM", ", dDHMM", ", dNmixture.\n")
 
 # Register the distributions explicitly for two reasons:
@@ -13,7 +13,7 @@
       BUGSdist = "dCJS_ss(probSurvive, probCapture, len)",
       Rdist = "dCJS_ss(probSurvive, probCapture, len = 0)",
       discrete = TRUE,
-      types = c('value = double(1)', 'probSurvive = double()', 'probCapture = double()', 'len = double()'),
+      types = c('value = double(1)', 'probSurvive = double()', 'probCapture = double()', 'len = integer()'),
       pqAvail = FALSE)), verbose = F
     )
 
@@ -22,7 +22,7 @@
       BUGSdist = "dCJS_sv(probSurvive, probCapture, len)",
       Rdist = "dCJS_sv(probSurvive, probCapture, len = 0)",
       discrete = TRUE,
-      types = c('value = double(1)', 'probSurvive = double()', 'probCapture = double(1)', 'len = double()'),
+      types = c('value = double(1)', 'probSurvive = double()', 'probCapture = double(1)', 'len = integer()'),
       pqAvail = FALSE)), verbose = F
     )
 
@@ -31,7 +31,7 @@
       BUGSdist = "dCJS_vs(probSurvive, probCapture, len)",
       Rdist = "dCJS_vs(probSurvive, probCapture, len = 0)",
       discrete = TRUE,
-      types = c('value = double(1)', 'probSurvive = double(1)', 'probCapture = double()', 'len = double()'),
+      types = c('value = double(1)', 'probSurvive = double(1)', 'probCapture = double()', 'len = integer()'),
       pqAvail = FALSE)), verbose = F
     )
 
@@ -40,7 +40,7 @@
       BUGSdist = "dCJS_vv(probSurvive, probCapture, len)",
       Rdist = "dCJS_vv(probSurvive, probCapture, len = 0)",
       discrete = TRUE,
-      types = c('value = double(1)', 'probSurvive = double(1)', 'probCapture = double(1)', 'len = double()'),
+      types = c('value = double(1)', 'probSurvive = double(1)', 'probCapture = double(1)', 'len = integer()'),
       pqAvail = FALSE)), verbose = F
   )
 
@@ -238,8 +238,8 @@
                 'init = double(1)',
                 'probObs = double(2)',
                 'probTrans = double(2)',
-                'len = double(0)',
-                'checkRowSums = double(0)'),
+                'len = integer(0)',
+                'checkRowSums = integer(0)'),
       mixedSizes = TRUE,
       pqAvail = FALSE)), verbose = F
     )
@@ -253,8 +253,8 @@
                 'init = double(1)',
                 'probObs = double(3)',
                 'probTrans = double(2)',
-                'len = double(0)',
-                'checkRowSums = double(0)'),
+                'len = integer(0)',
+                'checkRowSums = integer(0)'),
       mixedSizes = TRUE,
       pqAvail = FALSE)), verbose = F
     )
@@ -267,8 +267,8 @@
                 'init = double(1)',
                 'probObs = double(2)',
                 'probTrans = double(3)',
-                'len = double()',
-                'checkRowSums = double(0)'),
+                'len = integer()',
+                'checkRowSums = integer(0)'),
       mixedSizes = TRUE,
       pqAvail = FALSE)), verbose = F
     )
@@ -282,8 +282,8 @@
                 'init = double(1)',
                 'probObs = double(3)',
                 'probTrans = double(3)',
-                'len = double()',
-                'checkRowSums = double(0)'),
+                'len = integer()',
+                'checkRowSums = integer(0)'),
       mixedSizes = TRUE,
       pqAvail = FALSE)), verbose = F
     )
@@ -305,6 +305,23 @@
     )), verbose = F
   )
 
+  registerDistributions(list(
+    dNmixtureAD_v = list(
+      BUGSdist = "dNmixtureAD_v(lambda, prob, Nmin, Nmax, len)",
+      Rdist = "dNmixtureAD_v(lambda, prob, Nmin, Nmax, len)",
+      discrete = TRUE,
+      types = c('value = double(1)',
+                'lambda = double()',
+                'prob = double(1)',
+                'Nmin = double(0, default = -1)',
+                'Nmax = double(0, default = -1)',
+                'len = double()'
+                ),
+      mixedSizes = FALSE,
+      pqAvail = FALSE
+    )), verbose = F
+  )
+
   registerDistributions(list(
     dNmixture_s = list(
       BUGSdist = "dNmixture_s(lambda, prob, Nmin, Nmax, len)",
@@ -322,6 +339,24 @@
     )), verbose = F
   )
 
+  registerDistributions(list(
+    dNmixtureAD_s = list(
+      BUGSdist = "dNmixtureAD_s(lambda, prob, Nmin, Nmax, len)",
+      Rdist = "dNmixtureAD_s(lambda, prob, Nmin, Nmax, len)",
+      discrete = TRUE,
+      types = c('value = double(1)',
+                'lambda = double()',
+                'prob = double()',
+                'Nmin = double(0, default = -1)',
+                'Nmax = double(0, default = -1)',
+                'len = double()'
+                ),
+      mixedSizes = FALSE,
+      pqAvail = FALSE
+    )), verbose = F
+  )
+
+
   registerDistributions(list(
     dNmixture_BNB_v = list(
       BUGSdist = "dNmixture_BNB_v(lambda, theta, prob, Nmin, Nmax, len)",
@@ -339,6 +374,23 @@
       pqAvail = FALSE
     )), verbose = F
   )
+  registerDistributions(list(
+    dNmixtureAD_BNB_v = list(
+      BUGSdist = "dNmixtureAD_BNB_v(lambda, theta, prob, Nmin, Nmax, len)",
+      Rdist = "dNmixtureAD_BNB_v(lambda, theta, prob, Nmin, Nmax, len)",
+      discrete = TRUE,
+      types = c('value = double(1)',
+                'lambda = double()',
+                'theta = double()',
+                'prob = double(1)',
+                'Nmin = double(0, default = -1)',
+                'Nmax = double(0, default = -1)',
+                'len = double()'
+      ),
+      mixedSizes = FALSE,
+      pqAvail = FALSE
+    )), verbose = F
+  )
   registerDistributions(list(
     dNmixture_BNB_s = list(
       BUGSdist = "dNmixture_BNB_s(lambda, theta, prob, Nmin, Nmax, len)",
@@ -357,11 +409,11 @@
     )), verbose = F
   )
   registerDistributions(list(
-    dNmixture_BNB_oneObs = list(
-      BUGSdist = "dNmixture_BNB_oneObs(lambda, theta, prob, Nmin, Nmax, len)",
-      Rdist = "dNmixture_BNB_oneObs(lambda, theta, prob, Nmin, Nmax, len)",
+    dNmixtureAD_BNB_s = list(
+      BUGSdist = "dNmixtureAD_BNB_s(lambda, theta, prob, Nmin, Nmax, len)",
+      Rdist = "dNmixtureAD_BNB_s(lambda, theta, prob, Nmin, Nmax, len)",
       discrete = TRUE,
-      types = c('value = double()',
+      types = c('value = double(1)',
                 'lambda = double()',
                 'theta = double()',
                 'prob = double()',
@@ -373,6 +425,38 @@
       pqAvail = FALSE
     )), verbose = F
   )
+  registerDistributions(list(
+    dNmixture_BNB_oneObs = list(
+      BUGSdist = "dNmixture_BNB_oneObs(lambda, theta, prob, Nmin, Nmax)",
+      Rdist = "dNmixture_BNB_oneObs(lambda, theta, prob, Nmin, Nmax)",
+      discrete = TRUE,
+      types = c('value = double()',
+                'lambda = double()',
+                'theta = double()',
+                'prob = double()',
+                'Nmin = double(0, default = -1)',
+                'Nmax = double(0, default = -1)'
+      ),
+      mixedSizes = FALSE,
+      pqAvail = FALSE
+    )), verbose = F
+  )
+  registerDistributions(list(
+    dNmixtureAD_BNB_oneObs = list(
+      BUGSdist = "dNmixtureAD_BNB_oneObs(lambda, theta, prob, Nmin, Nmax)",
+      Rdist = "dNmixtureAD_BNB_oneObs(lambda, theta, prob, Nmin, Nmax)",
+      discrete = TRUE,
+      types = c('value = double()',
+                'lambda = double()',
+                'theta = double()',
+                'prob = double()',
+                'Nmin = double(0, default = -1)',
+                'Nmax = double(0, default = -1)'
+      ),
+      mixedSizes = FALSE,
+      pqAvail = FALSE
+    )), verbose = F
+  )
 
   registerDistributions(list(
     dNmixture_BBP_v = list(
@@ -391,6 +475,23 @@
       pqAvail = FALSE
     )), verbose = F
   )
+  registerDistributions(list(
+    dNmixtureAD_BBP_v = list(
+      BUGSdist = "dNmixtureAD_BBP_v(lambda, prob, s, Nmin, Nmax, len)",
+      Rdist = "dNmixtureAD_BBP_v(lambda, prob, s, Nmin, Nmax, len)",
+      discrete = TRUE,
+      types = c('value = double(1)',
+                'lambda = double()',
+                'prob = double(1)',
+                's = double()',
+                'Nmin = double(0, default = -1)',
+                'Nmax = double(0, default = -1)',
+                'len = double()'
+      ),
+      mixedSizes = FALSE,
+      pqAvail = FALSE
+    )), verbose = F
+  )
 
   registerDistributions(list(
     dNmixture_BBP_s = list(
@@ -409,11 +510,28 @@
       pqAvail = FALSE
     )), verbose = F
   )
+  registerDistributions(list(
+    dNmixtureAD_BBP_s = list(
+      BUGSdist = "dNmixtureAD_BBP_s(lambda, prob, s, Nmin, Nmax, len)",
+      Rdist = "dNmixtureAD_BBP_s(lambda, prob, s, Nmin, Nmax, len)",
+      discrete = TRUE,
+      types = c('value = double(1)',
+                'lambda = double()',
+                'prob = double()',
+                's = double()',
+                'Nmin = double(0, default = -1)',
+                'Nmax = double(0, default = -1)',
+                'len = double()'
+      ),
+      mixedSizes = FALSE,
+      pqAvail = FALSE
+    )), verbose = F
+  )
 
   registerDistributions(list(
     dNmixture_BBP_oneObs = list(
-      BUGSdist = "dNmixture_BBP_oneObs(lambda, prob, s, Nmin, Nmax, len)",
-      Rdist = "dNmixture_BBP_oneObs(lambda, prob, s, Nmin, Nmax, len)",
+      BUGSdist = "dNmixture_BBP_oneObs(lambda, prob, s, Nmin, Nmax)",
+      Rdist = "dNmixture_BBP_oneObs(lambda, prob, s, Nmin, Nmax)",
       discrete = TRUE,
       types = c('value = double()',
                 'lambda = double()',
@@ -427,6 +545,22 @@
       pqAvail = FALSE
     )), verbose = F
   )
+  registerDistributions(list(
+    dNmixtureAD_BBP_oneObs = list(
+      BUGSdist = "dNmixtureAD_BBP_oneObs(lambda, prob, s, Nmin, Nmax)",
+      Rdist = "dNmixtureAD_BBP_oneObs(lambda, prob, s, Nmin, Nmax)",
+      discrete = TRUE,
+      types = c('value = double()',
+                'lambda = double()',
+                'prob = double()',
+                's = double()',
+                'Nmin = double(0, default = -1)',
+                'Nmax = double(0, default = -1)'
+      ),
+      mixedSizes = FALSE,
+      pqAvail = FALSE
+    )), verbose = F
+  )
 
 
 
@@ -450,6 +584,25 @@
     )), verbose = F
   )
 
+  registerDistributions(list(
+    dNmixtureAD_BBNB_v = list(
+      BUGSdist = "dNmixtureAD_BBNB_v(lambda, theta, prob, s, Nmin, Nmax, len)",
+      Rdist = "dNmixtureAD_BBNB_v(lambda, theta, prob, s, Nmin, Nmax, len)",
+      discrete = TRUE,
+      types = c('value = double(1)',
+                'lambda = double()',
+                'prob = double(1)',
+                's = double()',
+                'theta = double()',
+                'Nmin = double(0, default = -1)',
+                'Nmax = double(0, default = -1)',
+                'len = double()'
+      ),
+      mixedSizes = FALSE,
+      pqAvail = FALSE
+    )), verbose = F
+  )
+
   registerDistributions(list(
     dNmixture_BBNB_s = list(
       BUGSdist = "dNmixture_BBNB_s(lambda, theta, prob, s, Nmin, Nmax, len)",
@@ -468,17 +621,16 @@
       pqAvail = FALSE
     )), verbose = F
   )
-
-    registerDistributions(list(
-    dNmixture_BBNB_oneObs = list(
-      BUGSdist = "dNmixture_BBNB_oneObs(lambda, theta, prob, s, Nmin, Nmax, len)",
-      Rdist = "dNmixture_BBNB_oneObs(lambda, theta, prob, s, Nmin, Nmax, len)",
+  registerDistributions(list(
+    dNmixtureAD_BBNB_s = list(
+      BUGSdist = "dNmixtureAD_BBNB_s(lambda, theta, prob, s, Nmin, Nmax, len)",
+      Rdist = "dNmixtureAD_BBNB_s(lambda, theta, prob, s, Nmin, Nmax, len)",
       discrete = TRUE,
-      types = c('value = double()',
+      types = c('value = double(1)',
                 'lambda = double()',
-                'theta = double()',
                 'prob = double()',
                 's = double()',
+                'theta = double()',
                 'Nmin = double(0, default = -1)',
                 'Nmax = double(0, default = -1)',
                 'len = double()'
@@ -488,16 +640,35 @@
     )), verbose = F
   )
 
-  registerDistributions(list(
-    dBetaBinom = list(
-      BUGSdist = "dBetaBinom(N, shape1, shape2)",
-      Rdist = "dBetaBinom(N, shape1, shape2)",
+    registerDistributions(list(
+    dNmixture_BBNB_oneObs = list(
+      BUGSdist = "dNmixture_BBNB_oneObs(lambda, theta, prob, s, Nmin, Nmax)",
+      Rdist = "dNmixture_BBNB_oneObs(lambda, theta, prob, s, Nmin, Nmax)",
       discrete = TRUE,
-      types = c('value = double(1)',
-                'N = double()',
-                'shape1 = double(1)',
-                'shape2 = double(1)',
-                'log = double()'
+      types = c('value = double()',
+                'lambda = double()',
+                'theta = double()',
+                'prob = double()',
+                's = double()',
+                'Nmin = double(0, default = -1)',
+                'Nmax = double(0, default = -1)'
+      ),
+      mixedSizes = FALSE,
+      pqAvail = FALSE
+    )), verbose = F
+  )
+    registerDistributions(list(
+    dNmixtureAD_BBNB_oneObs = list(
+      BUGSdist = "dNmixtureAD_BBNB_oneObs(lambda, theta, prob, s, Nmin, Nmax)",
+      Rdist = "dNmixtureAD_BBNB_oneObs(lambda, theta, prob, s, Nmin, Nmax)",
+      discrete = TRUE,
+      types = c('value = double()',
+                'lambda = double()',
+                'theta = double()',
+                'prob = double()',
+                's = double()',
+                'Nmin = double(0, default = -1)',
+                'Nmax = double(0, default = -1)'
       ),
       mixedSizes = FALSE,
       pqAvail = FALSE
@@ -505,15 +676,31 @@
   )
 
   registerDistributions(list(
-    dBetaBinom_One = list(
-      BUGSdist = "dBetaBinom_One(N, shape1, shape2)",
-      Rdist = "dBetaBinom_One(N, shape1, shape2)",
+    dBetaBinom_v = list(
+      BUGSdist = "dBetaBinom_v(N, shape1, shape2, len)",
+      Rdist = "dBetaBinom_v(N, shape1, shape2, len)",
       discrete = TRUE,
-      types = c('value = double()',
+      types = c('value = double(1)',
+                'N = double()',
+                'shape1 = double(1)',
+                'shape2 = double(1)',
+                'len = double()'
+      ),
+      mixedSizes = FALSE,
+      pqAvail = FALSE
+    )), verbose = F
+  )
+
+  registerDistributions(list(
+    dBetaBinom_s = list(
+      BUGSdist = "dBetaBinom_s(N, shape1, shape2, len)",
+      Rdist = "dBetaBinom_s(N, shape1, shape2, len)",
+      discrete = TRUE,
+      types = c('value = double(1)',
                 'N = double()',
                 'shape1 = double()',
                 'shape2 = double()',
-                'log = double()'
+                'len = double()'
       ),
       mixedSizes = FALSE,
       pqAvail = FALSE
