@@ -24,10 +24,26 @@ test_that("dNmixture_v works",
             correctProbX <- correctProbX + dpois(N, lambda) * prod(dbinom(x, N, prob))
           }
           expect_equal(probX, correctProbX)
+      # Manually calculate the correct answer with special-case Nmin and Nmax
+            Nmin <- 3
+            for(Nmax in 3:5) {
+              probX <- dNmixture_v(x, lambda, prob, Nmin, Nmax, len)
+              correctProbX <- 0
+              for (N in Nmin:Nmax) {
+                correctProbX <- correctProbX + dpois(N, lambda) * prod(dbinom(x, N, prob))
+              }
+              expect_equal(probX, correctProbX)
+            }
+            Nmin <- 0
+            Nmax <- 250
 
       # Uncompiled log probability
           lProbX <- dNmixture_v(x, lambda, prob, Nmin, Nmax, len, log = TRUE)
-          lCorrectProbX <- log(correctProbX)
+            correctProbX <- 0
+            for (N in Nmin:Nmax) {
+              correctProbX <- correctProbX + dpois(N, lambda) * prod(dbinom(x, N, prob))
+            }
+            lCorrectProbX <- log(correctProbX)
           expect_equal(lProbX, lCorrectProbX)
 
       # Dynamic Nmin / Nmax
@@ -42,13 +58,16 @@ test_that("dNmixture_v works",
 
       # Compilation and compiled calculations
           CdNmixture_v <- compileNimble(dNmixture_v)
-          CprobX <- CdNmixture_v(x, lambda, prob, Nmin, Nmax, len)
-          expect_equal(CprobX, probX)
+            CprobX <- CdNmixture_v(x, lambda, prob, Nmin, Nmax, len)
+            probX <- dNmixture_v(x, lambda, prob, Nmin, Nmax, len)
+            expect_equal(CprobX, probX)
 
           ClProbX <- CdNmixture_v(x, lambda, prob, Nmin, Nmax, len, log = TRUE)
-          expect_equal(ClProbX, lProbX)
+          lProbX <- dNmixture_v(x, lambda, prob, Nmin, Nmax, len, log=TRUE)
+            expect_equal(ClProbX, lProbX)
 
           CdynProbX <- CdNmixture_v(x, lambda, prob, Nmin = -1, Nmax = -1, len)
+          dynCorrectProbX <- dNmixture_v(x, lambda, prob, Nmin, Nmax, len)
           expect_equal(CdynProbX, dynCorrectProbX)
 
       # Use in Nimble model
@@ -295,17 +314,37 @@ test_that("dNmixture_BNB_v works",
             }
             expect_equal(dynProbX, dynCorrectProbX)
 
+            # Some special-case Nmax values
+            r <- 1 / theta
+            pNB <- 1 / (1 + theta * lambda)
+            Nmin <- 3
+            for(Nmax in 3:6) {
+              probX <- dNmixture_BNB_v(x, lambda, theta, prob, Nmin, Nmax, len)
+              # Manually calculate the correct answer
+              correctProbX <- 0
+              for (N in Nmin:Nmax) {
+                correctProbX <- correctProbX + dnbinom(N, size = r, prob = pNB) *
+                  prod(dbinom(x, N, prob))
+              }
+              expect_equal(probX, correctProbX)
+            }
+            Nmax <- 0
+            Nmax <- 250
 
             # Compilation and compiled calculations
             CdNmixture_BNB_v <- compileNimble(dNmixture_BNB_v)
             CprobX <- CdNmixture_BNB_v(x, lambda, theta, prob, Nmin, Nmax, len)
+            probX <- dNmixture_BNB_v(x, lambda, theta, prob, Nmin, Nmax, len)
             expect_equal(CprobX, probX)
 
             ClProbX <- CdNmixture_BNB_v(x, lambda, theta, prob, Nmin, Nmax, len, log = TRUE)
+            lProbX <- dNmixture_BNB_v(x, lambda, theta, prob, Nmin, Nmax, len, log = TRUE)
             expect_equal(ClProbX, lProbX)
 
             CdynProbX <- CdNmixture_BNB_v(x, lambda, theta, prob, Nmin = -1,
                                           Nmax = -1, len)
+            dynProbX <- dNmixture_BNB_v(x, lambda, theta, prob, Nmin = -1,
+                                          Nmax = -1, len)
             expect_equal(CdynProbX, dynCorrectProbX)
 
             # Use in Nimble model
@@ -532,9 +571,8 @@ test_that("dNmixture_BNB_oneObs works",
             prob <- c(0.5)
             Nmin <- 0
             Nmax <- 250
-            len <- 5
 
-            probX <- dNmixture_BNB_oneObs(x, lambda, theta, prob, Nmin, Nmax, len)
+            probX <- dNmixture_BNB_oneObs(x, lambda, theta, prob, Nmin, Nmax)
 
             # Manually calculate the correct answer
             r <- 1 / theta
@@ -548,12 +586,12 @@ test_that("dNmixture_BNB_oneObs works",
             expect_equal(probX, correctProbX)
 
             # Uncompiled log probability
-            lProbX <- dNmixture_BNB_oneObs(x, lambda, theta, prob, Nmin, Nmax, len, log = TRUE)
+            lProbX <- dNmixture_BNB_oneObs(x, lambda, theta, prob, Nmin, Nmax, log = TRUE)
             lCorrectProbX <- log(correctProbX)
             expect_equal(lProbX, lCorrectProbX)
 
             # Dynamic Nmin/Nmax
-            dynProbX <- dNmixture_BNB_oneObs(x, lambda, theta, prob, Nmin = -1, Nmax = -1, len)
+            dynProbX <- dNmixture_BNB_oneObs(x, lambda, theta, prob, Nmin = -1, Nmax = -1)
             dNmin <- 1; dNmax <- 17
             dynCorrectProbX <- 0
             for (N in dNmin:dNmax) {
@@ -563,18 +601,18 @@ test_that("dNmixture_BNB_oneObs works",
             expect_equal(dynProbX, dynCorrectProbX)
 
             CdynProbX <- dNmixture_BNB_oneObs(x, lambda, theta, prob, Nmin = -1,
-                                              Nmax = -1, len)
+                                              Nmax = -1)
             expect_equal(CdynProbX, dynCorrectProbX)
 
             # Compilation and compiled calculations
             CdNmixture_BNB_oneObs <- compileNimble(dNmixture_BNB_oneObs)
-            CprobX <- CdNmixture_BNB_oneObs(x, lambda, theta, prob, Nmin, Nmax, len)
+            CprobX <- CdNmixture_BNB_oneObs(x, lambda, theta, prob, Nmin, Nmax)
             expect_equal(CprobX, probX)
 
-            ClProbX <- CdNmixture_BNB_oneObs(x, lambda, theta, prob, Nmin, Nmax, len, log = TRUE)
+            ClProbX <- CdNmixture_BNB_oneObs(x, lambda, theta, prob, Nmin, Nmax, log = TRUE)
             expect_equal(ClProbX, lProbX)
 
-            CdynProbX <- CdNmixture_BNB_oneObs(x, lambda, theta, prob, Nmin = -1, Nmax = -1, len)
+            CdynProbX <- CdNmixture_BNB_oneObs(x, lambda, theta, prob, Nmin = -1, Nmax = -1)
             expect_equal(CdynProbX, dynCorrectProbX)
 
 
@@ -582,8 +620,7 @@ test_that("dNmixture_BNB_oneObs works",
             nc <- nimbleCode({
               x ~ dNmixture_BNB_oneObs(lambda = lambda, prob = prob,
                                        theta = theta,
-                                       Nmin = Nmin, Nmax = Nmax, len = len)
-
+                                       Nmin = Nmin, Nmax = Nmax)
             })
 
             m <- nimbleModel(code = nc,
@@ -591,8 +628,7 @@ test_that("dNmixture_BNB_oneObs works",
                              inits = list(lambda = lambda,
                                           prob = prob,
                                           theta = theta),
-                             constants = list(Nmin = Nmin, Nmax = Nmax,
-                                              len = len))
+                             constants = list(Nmin = Nmin, Nmax = Nmax))
             m$calculate()
             MlProbX <- m$getLogProb("x")
             expect_equal(MlProbX, lProbX)
@@ -609,8 +645,7 @@ test_that("dNmixture_BNB_oneObs works",
                                inits = list(lambda = lambda,
                                             theta = theta,
                                             prob = prob),
-                               constants = list(Nmin = Nmin, Nmax = Nmax,
-                                                len = len))
+                               constants = list(Nmin = Nmin, Nmax = Nmax))
 
 
             mNAConf <- configureMCMC(mNA)
@@ -629,14 +664,14 @@ test_that("dNmixture_BNB_oneObs works",
             xSim <- numeric(nSim)
             set.seed(1)
             for (i in 1:nSim) {
-              xSim[i] <- rNmixture_BNB_oneObs(1, lambda, theta, prob, Nmin, Nmax, len)
+              xSim[i] <- rNmixture_BNB_oneObs(1, lambda, theta, prob, Nmin, Nmax)
             }
 
             CrNmixture_BNB_oneObs <- compileNimble(rNmixture_BNB_oneObs)
             CxSim <- numeric(nSim)
             set.seed(1)
             for (i in 1:nSim) {
-              CxSim[i] <- CrNmixture_BNB_oneObs(1, lambda, theta, prob, Nmin, Nmax, len)
+              CxSim[i] <- CrNmixture_BNB_oneObs(1, lambda, theta, prob, Nmin, Nmax)
             }
             expect_identical(xSim, CxSim)
 
@@ -681,7 +716,7 @@ test_that("dNmixture_BBP_v works",
             correctProbX <- 0
             for (N in Nmin:Nmax) {
               correctProbX <- correctProbX + dpois(N, lambda) *
-                prod(dBetaBinom(x, N, shape1 = alpha, shape2 = beta))
+                prod(dBetaBinom_v(x, N, shape1 = alpha, shape2 = beta))
             }
 
             expect_equal(probX, correctProbX)
@@ -824,8 +859,8 @@ test_that("dNmixture_BBP_s works",
             correctProbX <- 0
             for (N in Nmin:Nmax) {
               correctProbX <- correctProbX + dpois(N, lambda) *
-                prod(dBetaBinom(x, N,
-                                shape1 = rep(alpha, len), shape2 = rep(beta, len)))
+                prod(dBetaBinom_s(x, N,
+                                  alpha, shape2 = beta))
             }
 
             expect_equal(probX, correctProbX)
@@ -959,7 +994,7 @@ test_that("dNmixture_BBP_oneObs works",
             Nmax <- 250
             len <- 5
 
-            probX <- dNmixture_BBP_oneObs(x, lambda, prob, s, Nmin, Nmax, len)
+            probX <- dNmixture_BBP_oneObs(x, lambda, prob, s, Nmin, Nmax)
 
             # Manually calculate the correct answer
             alpha <- prob * s
@@ -967,49 +1002,49 @@ test_that("dNmixture_BBP_oneObs works",
             correctProbX <- 0
             for (N in Nmin:Nmax) {
               correctProbX <- correctProbX + dpois(N, lambda) *
-                prod(dBetaBinom_One(x, N, alpha, beta))
+                prod(dBetaBinom_s(x, N, alpha, beta))
             }
 
             expect_equal(probX, correctProbX)
 
             # Uncompiled log probability
-            lProbX <- dNmixture_BBP_oneObs(x, lambda, prob, s, Nmin, Nmax, len, log = TRUE)
+            lProbX <- dNmixture_BBP_oneObs(x, lambda, prob, s, Nmin, Nmax, log = TRUE)
             lCorrectProbX <- log(correctProbX)
             expect_equal(lProbX, lCorrectProbX)
 
             # Compilation and compiled calculations
             CdNmixture_BBP_oneObs <- compileNimble(dNmixture_BBP_oneObs)
-            CprobX <- CdNmixture_BBP_oneObs(x, lambda, prob, s, Nmin, Nmax, len)
+            CprobX <- CdNmixture_BBP_oneObs(x, lambda, prob, s, Nmin, Nmax)
             expect_equal(CprobX, probX)
 
-            ClProbX <- CdNmixture_BBP_oneObs(x, lambda, prob, s, Nmin, Nmax, len, log = TRUE)
+            ClProbX <- CdNmixture_BBP_oneObs(x, lambda, prob, s, Nmin, Nmax, log = TRUE)
             expect_equal(ClProbX, lProbX)
 
             # Dynamic Nmin / Nmax isn't allowed
             expect_error({
-              dNmixture_BBP_oneObs(x, lambda, prob, s, Nmin = -1, Nmax = -1, len)
+              dNmixture_BBP_oneObs(x, lambda, prob, s, Nmin = -1, Nmax = -1)
             })
             expect_error({
-              dNmixture_BBP_oneObs(x, lambda, prob, s, Nmin = -1, Nmax, len)
+              dNmixture_BBP_oneObs(x, lambda, prob, s, Nmin = -1, Nmax)
             })
             expect_error({
-              dNmixture_BBP_oneObs(x, lambda, prob, s, Nmin, Nmax = -1, len)
+              dNmixture_BBP_oneObs(x, lambda, prob, s, Nmin, Nmax = -1)
             })
             expect_error({
-              CdNmixture_BBP_oneObs(x, lambda, prob, s, Nmin = -1, Nmax = -1, len)
+              CdNmixture_BBP_oneObs(x, lambda, prob, s, Nmin = -1, Nmax = -1)
             })
             expect_error({
-              CdNmixture_BBP_oneObs(x, lambda, prob, s, Nmin = -1, Nmax, len)
+              CdNmixture_BBP_oneObs(x, lambda, prob, s, Nmin = -1, Nmax)
             })
             expect_error({
-              CdNmixture_BBP_oneObs(x, lambda, prob, s, Nmin, Nmax = -1, len)
+              CdNmixture_BBP_oneObs(x, lambda, prob, s, Nmin, Nmax = -1)
             })
 
             # Use in Nimble model
             nc <- nimbleCode({
               x ~ dNmixture_BBP_oneObs(lambda = lambda, prob = prob,
                                   s = s,
-                                  Nmin = Nmin, Nmax = Nmax, len = len)
+                                  Nmin = Nmin, Nmax = Nmax)
 
             })
 
@@ -1018,8 +1053,7 @@ test_that("dNmixture_BBP_oneObs works",
                              inits = list(lambda = lambda,
                                           prob = prob,
                                           s = s),
-                             constants = list(Nmin = Nmin, Nmax = Nmax,
-                                              len = len))
+                             constants = list(Nmin = Nmin, Nmax = Nmax))
             m$calculate()
             MlProbX <- m$getLogProb("x")
             expect_equal(MlProbX, lProbX)
@@ -1036,8 +1070,7 @@ test_that("dNmixture_BBP_oneObs works",
                                inits = list(lambda = lambda,
                                             s = s,
                                             prob = prob),
-                               constants = list(Nmin = Nmin, Nmax = Nmax,
-                                                len = len))
+                               constants = list(Nmin = Nmin, Nmax = Nmax))
 
 
             mNAConf <- configureMCMC(mNA)
@@ -1056,14 +1089,14 @@ test_that("dNmixture_BBP_oneObs works",
             xSim <- numeric(nSim)
             set.seed(1)
             for (i in 1:nSim) {
-              xSim[i] <- rNmixture_BBP_oneObs(1, lambda, prob, s, Nmin, Nmax, len)
+              xSim[i] <- rNmixture_BBP_oneObs(1, lambda, prob, s, Nmin, Nmax)
             }
 
             CrNmixture_BBP_oneObs <- compileNimble(rNmixture_BBP_oneObs)
             CxSim <- numeric(nSim)
             set.seed(1)
             for (i in 1:nSim) {
-              CxSim[i] <- CrNmixture_BBP_oneObs(1, lambda, prob, s, Nmin, Nmax, len)
+              CxSim[i] <- CrNmixture_BBP_oneObs(1, lambda, prob, s, Nmin, Nmax)
             }
             expect_identical(xSim, CxSim)
 
@@ -1113,7 +1146,7 @@ test_that("dNmixture_BBNB_v works",
             correctProbX <- 0
             for (N in Nmin:Nmax) {
               correctProbX <- correctProbX + dnbinom(N, size = r, prob = pNB) *
-                prod(dBetaBinom(x, N, shape1 = alpha, shape2 = beta))
+                prod(dBetaBinom_v(x, N, shape1 = alpha, shape2 = beta))
             }
 
             expect_equal(probX, correctProbX)
@@ -1256,7 +1289,7 @@ test_that("dNmixture_BBNB_s works",
             correctProbX <- 0
             for (N in Nmin:Nmax) {
               correctProbX <- correctProbX + dnbinom(N, size = r, prob = pNB) *
-                prod(dBetaBinom(x, N, shape1 = rep(alpha, len), shape2 = rep(beta, len)))
+                prod(dBetaBinom_s(x, N, shape1 = alpha, shape2 = beta))
             }
 
             expect_equal(probX, correctProbX)
@@ -1388,7 +1421,7 @@ test_that("dNmixture_BBNB_oneObs works",
             Nmax <- 250
             len <- 1
 
-            probX <- dNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin, Nmax, len)
+            probX <- dNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin, Nmax)
 
             # Manually calculate the correct answer
             alpha <- prob * s
@@ -1398,49 +1431,49 @@ test_that("dNmixture_BBNB_oneObs works",
             correctProbX <- 0
             for (N in Nmin:Nmax) {
               correctProbX <- correctProbX + dnbinom(N, size = r, prob = pNB) *
-                prod(dBetaBinom_One(x, N, alpha, beta))
+                prod(dBetaBinom_s(x, N, alpha, beta))
             }
 
             expect_equal(probX, correctProbX)
 
             # Uncompiled log probability
-            lProbX <- dNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin, Nmax, len, log = TRUE)
+            lProbX <- dNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin, Nmax, log = TRUE)
             lCorrectProbX <- log(correctProbX)
             expect_equal(lProbX, lCorrectProbX)
 
             # Compilation and compiled calculations
             CdNmixture_BBNB_oneObs <- compileNimble(dNmixture_BBNB_oneObs)
-            CprobX <- CdNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin, Nmax, len)
+            CprobX <- CdNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin, Nmax)
             expect_equal(CprobX, probX)
 
-            ClProbX <- CdNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin, Nmax, len, log = TRUE)
+            ClProbX <- CdNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin, Nmax, log = TRUE)
             expect_equal(ClProbX, lProbX)
 
             # Dynamic Nmin / Nmax isn't allowed
             expect_error({
-              dNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin = -1, Nmax = -1, len)
+              dNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin = -1, Nmax = -1)
             })
             expect_error({
-              dNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin, Nmax = -1, len)
+              dNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin, Nmax = -1)
             })
             expect_error({
-              dNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin = -1, Nmax, len)
+              dNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin = -1, Nmax)
             })
             expect_error({
-              CdNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin = -1, Nmax = -1, len)
+              CdNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin = -1, Nmax = -1)
             })
             expect_error({
-              CdNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin, Nmax = -1, len)
+              CdNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin, Nmax = -1)
             })
             expect_error({
-              CdNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin = -1, Nmax, len)
+              CdNmixture_BBNB_oneObs(x, lambda, theta, prob, s, Nmin = -1, Nmax)
             })
 
             # Use in Nimble model
             nc <- nimbleCode({
               x ~ dNmixture_BBNB_oneObs(lambda = lambda, prob = prob,
                                   s = s, theta = theta,
-                                  Nmin = Nmin, Nmax = Nmax, len = len)
+                                  Nmin = Nmin, Nmax = Nmax)
 
             })
 
@@ -1450,8 +1483,7 @@ test_that("dNmixture_BBNB_oneObs works",
                                           prob = prob,
                                           theta = theta,
                                           s = s),
-                             constants = list(Nmin = Nmin, Nmax = Nmax,
-                                              len = len))
+                             constants = list(Nmin = Nmin, Nmax = Nmax))
             m$calculate()
             MlProbX <- m$getLogProb("x")
             expect_equal(MlProbX, lProbX)
@@ -1468,8 +1500,7 @@ test_that("dNmixture_BBNB_oneObs works",
                                inits = list(lambda = lambda,
                                             s = s, theta = theta,
                                             prob = prob),
-                               constants = list(Nmin = Nmin, Nmax = Nmax,
-                                                len = len))
+                               constants = list(Nmin = Nmin, Nmax = Nmax))
 
 
             mNAConf <- configureMCMC(mNA)
@@ -1488,14 +1519,14 @@ test_that("dNmixture_BBNB_oneObs works",
             xSim <- numeric(nSim)
             set.seed(1)
             for (i in 1:nSim) {
-              xSim[i] <- rNmixture_BBNB_oneObs(1, lambda, theta, prob, s, Nmin, Nmax, len)
+              xSim[i] <- rNmixture_BBNB_oneObs(1, lambda, theta, prob, s, Nmin, Nmax)
             }
 
             CrNmixture_BBNB_oneObs <- compileNimble(rNmixture_BBNB_oneObs)
             CxSim <- numeric(nSim)
             set.seed(1)
             for (i in 1:nSim) {
-              CxSim[i] <- CrNmixture_BBNB_oneObs(1, lambda, theta, prob, s, Nmin, Nmax, len)
+              CxSim[i] <- CrNmixture_BBNB_oneObs(1, lambda, theta, prob, s, Nmin, Nmax)
             }
             expect_identical(xSim, CxSim)
 
